;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%;
; crash to crash turbo v0.2       20/mar/99              lord_nexus@usa.net ;
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%;
;  usage        : run to toggle installed state, denoted by <i> symbol. 
;                 once installed the isr will catch the following combos,   ;
;                    on + y=       crash to crash                           ;
;                    on + window   turbo normal                             ;
;                    on + zoom     turbo medium                             ;
;                    on + trace    turbo fast                               ;
;                    on + graph    turbo fastest                            ;
;                                                                           ;
;  notes        : remember that on crash enrty/exit turbo is set normal     ;
;               : on apd calc will do 'standard off' rather than 'apd off'  ;
;                                                                           ;
;  apd_layout   : 0x8228  0x15b  free                                       ;
;                 0x8383  ...    int_start                                  ;
;                 0x8400  0x100  interrupt vector table                     ;
;                                                                           ;
;  credit       : crcrash, ld2train                                         ;
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%;

INT_TURBO       .equ            $c1

                .include        "crash82.inc"
z_name          .db             "crash to crash turbo v0.2"
z_name_inst     .db             $00
                .db             " <i>",$00


;===========================================================================;
; main                                                                      ;
;===========================================================================;
main            ld              hl,z_name_inst

                ld              a,(INT_STATE)
                cp              INT_TURBO
                jr              z,remove
	
                or              a
                ret             nz                      ; if another isr ret


;---------------------------------------------------------------------------;
; install interrupt                                                         ;
;---------------------------------------------------------------------------;
                ld              (hl),$20

                ld              hl,$8400
                ld              de,$8401
                ld              (hl),$83                ; int_handle = 0x8383
                ld              bc,$100
                ldir                          
	
                ld              hl,int_start
                ld              de,$8383                    
                ld              bc,int_end-int_start+1
                ldir
	
                ld              hl,$15b                 ; 0x15b free
                ld              (APD_FREE),hl
                
                ld              a,$84                   ; int table at 0x8400
                ld              i,a
	
                im              2
	
                ret
;===========================================================================;


;===========================================================================;
; remove interrupt                                                          ;
;===========================================================================;
remove          ld              (hl),$00

                im              1
	
                ld              hl,$300                 ; apd all free
                ld              (APD_FREE),hl
	
                xor             a
                ld              i,a

                ld              (INT_STATE),a
		
                ret
;===========================================================================;


;===========================================================================;
; interrupt handler                                                         ;
;===========================================================================;
int_start       di
                ex              af,af'
                exx

                ld              a,INT_TURBO
                ld              (INT_STATE),a

                ld              a,($800a)               ; apd countdown lsb
                cp              $11
                jr              c,tioff                 ; when lsb<$11 tioff

int_main        in              a,(P_STATUS)
                bit             3,a
                jr              nz,int_exit             ; K_ON

                ld              a,%10111111             ; kb function mask
                out             (P_KEYBOARD),a
                in              a,(P_KEYBOARD)

                rra
                jr              nc,tbo_fastest          ; K_GRAPH

                rra
                jr              nc,tbo_fast             ; K_TRACE

                rra
                jr              nc,tbo_medium           ; K_ZOOM

                rra                 
                jr              nc,tbo_normal           ; K_WINDOW

                rra
                jr              c,int_exit              ; K_Y

;---------------------------------------------------------------------------;
; crash2crash
;---------------------------------------------------------------------------;
crash2crash     ld              a,($8121)
                cp              $40
                jr              nz,int_exit_on          ; is? home screen

                bit             0,(iy+$12)
                jr              nz,int_exit_on          ; is? no busy indicator

                bit             4,(iy+$0c)
                jr              nz,int_exit_on          ; is? no menu/cursor lock

                ld              hl,$8d4e                
                ex              (sp),hl                 ; set iret to crash entry
;---------------------------------------------------------------------------;

int_exit_on     res             4,(iy+$09)              ; reset on state
int_exit        exx
                ex              af,af'
                jp              $38                     ; original handler

;---------------------------------------------------------------------------;
; tioff
;---------------------------------------------------------------------------;
tioff           ld              hl,$058c                
                ex              (sp),hl                 ; set iret to 2nd-off
                jr              int_exit_on
;---------------------------------------------------------------------------;

;---------------------------------------------------------------------------;
; tbo fastest/fast/medium/normal
;---------------------------------------------------------------------------;
tbo_fastest     in              a,(P_KEYBOARD)          ;
                rra                                     ;
                jr              nc,tbo_fastest          ; release K_GRAPH
                ld              a,%00010000
                jr              tbo_set

tbo_fast        in              a,(P_KEYBOARD)          ;
                bit             1,a                     ;
                jr              z,tbo_fast              ; release K_TRACE
                ld              a,%00010010
                jr              tbo_set

tbo_medium      in              a,(P_KEYBOARD)          ;
                bit             2,a                     ;
                jr              z,tbo_medium            ; release K_ZOOM
                ld              a,%00010100
                jr              tbo_set

tbo_normal      in              a,(P_KEYBOARD)          ;
                bit             3,a                     ;
                jr              z,tbo_normal            ; release K_WINDOW
                ld              a,%00010110

tbo_set         out             (P_INT),a
                jr              int_exit_on
;---------------------------------------------------------------------------;

int_end
;===========================================================================;
